@@ -142,9 +142,11 @@ void KFileFilterCombo::setFilters(const QList<KFileFilter> &types, const KFileFi
     }
 
     if (d->m_allTypes) {
-        QStringList allTypes;
+        QStringList allMimePatterns;
+        QStringList allFilePatterns;
         for (const KFileFilter &filter : std::as_const(d->m_filters)) {
-            allTypes << filter.mimePatterns().join(QLatin1Char(' '));
+            allMimePatterns << filter.mimePatterns();
+            allFilePatterns << filter.filePatterns();
         }
 
         KFileFilter allSupportedFilesFilter;
@@ -155,9 +157,9 @@ void KFileFilterCombo::setFilters(const QList<KFileFilter> &types, const KFileFi
                 allComments << filter.label();
             }
 
-            allSupportedFilesFilter = KFileFilter(allComments.join(delim), {}, allTypes);
+            allSupportedFilesFilter = KFileFilter(allComments.join(delim), allFilePatterns, allMimePatterns);
         } else {
-            allSupportedFilesFilter = KFileFilter(i18n("All Supported Files"), {}, allTypes);
+            allSupportedFilesFilter = KFileFilter(i18n("All Supported Files"), allFilePatterns, allMimePatterns);
             d->m_hasAllSupportedFiles = true;
         }
 
@@ -168,7 +170,14 @@ void KFileFilterCombo::setFilters(const QList<KFileFilter> &types, const KFileFi
 
     if (hasAllFilesFilter) {
         addItem(i18n("All Files"));
-        d->m_filters.append(KFileFilter(i18n("All Files"), {}, {QStringLiteral("application/octet-stream")}));
+
+        KFileFilter allFilter(i18n("All Files"), {}, {QStringLiteral("application/octet-stream")});
+
+        d->m_filters.append(allFilter);
+
+        if (defaultFilter == allFilter) {
+            setCurrentIndex(count() - 1);
+        }
     }
 
     d->m_lastFilter = currentText();
@@ -210,7 +219,7 @@ void KFileFilterCombo::setCurrentFilter(const KFileFilter &filter)
     auto it = std::find(d->m_filters.cbegin(), d->m_filters.cend(), filter);
 
     if (it == d->m_filters.cend()) {
-        qCWarning(KIO_KFILEWIDGETS_KFILEFILTERCOMBO) << "Could not find file filter";
+        qCWarning(KIO_KFILEWIDGETS_KFILEFILTERCOMBO) << "KFileFilterCombo::setCurrentFilter: Could not find file filter" << filter;
         setCurrentIndex(-1);
         Q_EMIT filterChanged();
         return;
