@@ -17,6 +17,7 @@
 #include <QApplication>
 #include <QDir>
 #include <QKeyEvent>
+#include <QLibraryInfo>
 #include <QMenu>
 #include <QMetaMethod>
 #include <QMimeData>
@@ -51,6 +52,7 @@
 
 #include <chrono>
 #include <cmath>
+#include <set>
 
 #include "kfileplaceeditdialog.h"
 #include "kfileplacesmodel.h"
@@ -102,6 +104,7 @@ void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
     painter->save();
 
     QStyleOptionViewItem opt = option;
+    const QPersistentModelIndex persistentIndex(index);
 
     const KFilePlacesModel *placesModel = static_cast<const KFilePlacesModel *>(index.model());
 
@@ -227,7 +230,6 @@ void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
     }
 
     if (placesModel->data(index, KFilePlacesModel::CapacityBarRecommendedRole).toBool()) {
-        QPersistentModelIndex persistentIndex(index);
         const auto info = m_freeSpaceInfo.value(persistentIndex);
 
         checkFreeSpace(index); // async
@@ -280,9 +282,18 @@ void KFilePlacesViewDelegate::paint(QPainter *painter, const QStyleOptionViewIte
         }
     }
 
-    painter->drawText(rectText,
-                      Qt::AlignLeft | Qt::AlignVCenter,
-                      opt.fontMetrics.elidedText(index.model()->data(index).toString(), Qt::ElideRight, rectText.width()));
+    const QString text = index.model()->data(index).toString();
+    const QString elidedText = opt.fontMetrics.elidedText(text, Qt::ElideRight, rectText.width());
+
+    const bool isElided = (text != elidedText);
+
+    if (isElided) {
+        m_elidedTexts.insert(persistentIndex);
+    } else if (auto it = m_elidedTexts.find(persistentIndex); it != m_elidedTexts.end()) {
+        m_elidedTexts.erase(it);
+    }
+
+    painter->drawText(rectText, Qt::AlignLeft | Qt::AlignVCenter, elidedText);
 
     painter->restore();
 }
@@ -309,6 +320,57 @@ bool KFilePlacesViewDelegate::helpEvent(QHelpEvent *event, QAbstractItemView *vi
                     return true;
                 }
             }
+        } else if (pointIsHeaderArea(event->pos())) {
+            // Make sure the tooltip doesn't linger when moving the mouse to the header area
+            // TODO show section name in a tooltip, too, if it is elided.
+            QToolTip::hideText();
+            event->setAccepted(true);
+            return true;
+        } else {
+            const bool isElided = m_elidedTexts.find(QPersistentModelIndex(index)) != m_elidedTexts.end();
+
+            const QString displayText = index.data(Qt::DisplayRole).toString();
+            QString toolTipText = index.data(Qt::ToolTipRole).toString();
+
+            if (isElided) {
+                if (!toolTipText.isEmpty()) {
+                    toolTipText = i18nc("@info:tooltip full display text since it is elided: original tooltip", "%1: %2", displayText, toolTipText);
+                } else {
+                    toolTipText = displayText;
+                }
+            }
+
+            if (index.data(KFilePlacesModel::CapacityBarRecommendedRole).toBool()) {
+                const auto info = m_freeSpaceInfo.value(index);
+
+                if (info.size > 0) {
+                    const quint64 available = info.size - info.used;
+                    const int percentUsed = qRound(100.0 * qreal(info.used) / qreal(info.size));
+
+                    if (!toolTipText.isEmpty()) {
+                        toolTipText.append(QLatin1Char('\n'));
+                    }
+                    toolTipText.append(i18nc("Available space out of total partition size (percent used)",
+                                             "%1 free of %2 (%3% used)",
+                                             KIO::convertSize(available),
+                                             KIO::convertSize(info.size),
+                                             percentUsed));
+                }
+            }
+
+            if (!toolTipText.isEmpty()) {
+                // FIXME remove once we depend on Qt 6.8
+                // Qt Wayland before 6.8 doesn't support popup repositioning, causing the tooltips
+                // remain stuck in place which is distracting.
+                static bool canRepositionPopups =
+                    !qApp->platformName().startsWith(QLatin1String("wayland")) || QLibraryInfo::version() >= QVersionNumber(6, 8, 0);
+                if (canRepositionPopups) {
+                    QToolTip::showText(event->globalPos(), toolTipText, m_view, m_view->visualRect(index));
+                }
+                // Always accepting the event to make sure QAbstractItemDelegate doesn't show it for us.
+                event->setAccepted(true);
+                return true;
+            }
         }
     }
     return QAbstractItemDelegate::helpEvent(event, view, option, index);
@@ -641,6 +703,7 @@ QColor KFilePlacesViewDelegate::mixedColor(const QColor &c1, const QColor &c2, i
 
 int KFilePlacesViewDelegate::sectionHeaderHeight(const QModelIndex &index) const
 {
+    Q_UNUSED(index);
     // Account for the spacing between header and item
     const int spacing = (s_lateralMargin + m_view->spacing());
     int height = m_view->fontMetrics().height() + spacing;
@@ -975,7 +1038,7 @@ void KFilePlacesView::setUrl(const QUrl &url)
 
         d->m_currentUrl = url;
 
-        if (placesModel->url(index) == url.adjusted(QUrl::StripTrailingSlash)) {
+        if (placesModel->url(index).matches(url, QUrl::StripTrailingSlash)) {
             selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
         } else {
             selectionModel()->clear();
@@ -1707,19 +1770,44 @@ void KFilePlacesViewPrivate::adaptItemSize()
 
     const int minSize = q->style()->pixelMetric(QStyle::PM_SmallIconSize);
     const int maxSize = 64;
-
-    int textWidth = 0;
     QFontMetrics fm = q->fontMetrics();
+
+    /// A set that contains the integer widths of the rows of the widest rows.
+    /// It is supposed to only contain the widest 20% of the rows. A std::set of integers is always sorted in ascending order.
+    std::set<int> widestTwentyPercentOfTextWidths;
+
+    /// The number of items that should be in widestTwentyPercentOfTextWidths.
+    const unsigned int twentyPercent = static_cast<int>(std::ceil(0.2 * rowCount));
+    Q_ASSERT(twentyPercent > 0); // This is an expectation of the code below or it might crash.
+
+    /// In this for-loop widestTwentyPercentOfTextWidths is filled, so it becomes true to its name.
+    /// Example: If there are rows with widths of 36, 28, 21, 35, 67, 70, 56, 41, 48, 178, then twentyPercent should equal 2,
+    ///          and after this for-loop widestTwentyPercentOfTextWidths should only consist of 70 and 178.
     for (int i = 0; i < placesModel->rowCount(); ++i) {
         QModelIndex index = placesModel->index(i, 0);
 
         if (!placesModel->isHidden(index)) {
-            textWidth = qMax(textWidth, fm.boundingRect(index.data(Qt::DisplayRole).toString()).width());
+            const int textWidth = fm.boundingRect(index.data(Qt::DisplayRole).toString()).width();
+            if (widestTwentyPercentOfTextWidths.size() < twentyPercent || textWidth > (*widestTwentyPercentOfTextWidths.begin())) {
+                // begin() is the smallest integer in the std::set.
+                if (widestTwentyPercentOfTextWidths.size() + 1 > twentyPercent) {
+                    // We need to remove an integer from the set, so the total count does not grow beyond our target size of twentyPercent.
+                    widestTwentyPercentOfTextWidths.erase(widestTwentyPercentOfTextWidths.begin());
+                }
+                widestTwentyPercentOfTextWidths.insert(textWidth);
+            }
         }
     }
 
     const int margin = q->style()->pixelMetric(QStyle::PM_FocusFrameHMargin, nullptr, q) + 1;
-    const int maxWidth = q->viewport()->width() - textWidth - 4 * margin - 1;
+    int maxWidth; /// The maximum width the icons should have based on the text widths. If a lot of text is cut off, we use a smaller icon.
+    if ((*widestTwentyPercentOfTextWidths.rbegin()) - (*widestTwentyPercentOfTextWidths.begin()) > 40) {
+        // There is quite a difference in width if we ignore the widest 20% of texts so let's ignore them.
+        // This makes it more likely that bigger icons are chosen if there is also enough vertical space.
+        maxWidth = q->viewport()->width() - (*widestTwentyPercentOfTextWidths.begin()) - 4 * margin - 1;
+    } else {
+        maxWidth = q->viewport()->width() - (*widestTwentyPercentOfTextWidths.rbegin()) - 4 * margin - 1;
+    }
 
     const int totalItemsHeight = (fm.height() / 2) * rowCount;
     const int totalSectionsHeight = m_delegate->sectionHeaderHeight(QModelIndex()) * sectionsCount();
