@@ -434,7 +434,9 @@ bool TrashImpl::move(const QString &src, const QString &dest)
     if (directRename(src, dest)) {
         // This notification is done by KIO::moveAs when using the code below
         // But if we do a direct rename we need to do the notification ourselves
+#ifdef WITH_QTDBUS
         org::kde::KDirNotify::emitFilesAdded(QUrl::fromLocalFile(dest));
+#endif
         return true;
     }
     if (m_lastErrorCode != KIO::ERR_UNSUPPORTED_ACTION) {
@@ -781,8 +783,10 @@ bool TrashImpl::readInfoFile(const QString &infoPath, TrashedFileInfo &info, int
     if (trashId == 0) {
         Q_ASSERT(info.origPath[0] == QLatin1Char('/'));
     } else {
-        const QString topdir = topDirectoryPath(trashId); // includes trailing slash
-        info.origPath.prepend(topdir);
+        if (!info.origPath.startsWith(QLatin1Char('/'))) {
+            const QString topdir = topDirectoryPath(trashId); // includes trailing slash
+            info.origPath.prepend(topdir);
+        }
     }
     const QString line = group.readEntry("DeletionDate");
     if (!line.isEmpty()) {
@@ -855,7 +859,9 @@ void TrashImpl::fileRemoved()
         KConfigGroup group = m_config.group(QStringLiteral("Status"));
         group.writeEntry("Empty", true);
         m_config.sync();
+#ifdef WITH_QTDBUS
         org::kde::KDirNotify::emitFilesChanged({QUrl::fromEncoded("trash:/")});
+#endif
     }
     // The apps showing the trash (e.g. kdesktop) will be notified
     // of this change when KDirNotify::FilesRemoved(...) is emitted,
